package com.ces.framework.config;

import com.ces.framework.shiro.filter.CustomShiroFilterFactoryBean;
import com.ces.framework.shiro.realm.UserRealm;
import org.apache.shiro.mgt.SecurityManager;
import org.apache.shiro.session.mgt.eis.EnterpriseCacheSessionDAO;
import org.apache.shiro.spring.security.interceptor.AuthorizationAttributeSourceAdvisor;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.apache.shiro.web.session.mgt.DefaultWebSessionManager;
import org.springframework.aop.framework.autoproxy.DefaultAdvisorAutoProxyCreator;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.LinkedHashMap;

/**
 * shiro配置。
 * 
 * @author khaoth
 */
@Configuration
public class ShiroConfig {

    /**
     * 身份和权限验证的realm。
     * 
     * @return
     */
    @Bean
    public UserRealm userRealm() {
        return new UserRealm();
    }

    /**
     * shiro的session管理器。
     * 
     * @return
     */
    @Bean
    public DefaultWebSessionManager sessionManager() {

        DefaultWebSessionManager sessionManager = new DefaultWebSessionManager();
        sessionManager.setSessionDAO(new EnterpriseCacheSessionDAO());
        // 60000毫秒等于1分钟。
        sessionManager.setGlobalSessionTimeout(-1000L);
        sessionManager.setSessionValidationInterval(1800000);
        // 去除url中的sessionId。
        sessionManager.setSessionIdUrlRewritingEnabled(false);
        return sessionManager;
    }

    /**
     * 配置securityManager中的参数。
     *
     * @return
     */
    @Bean
    public DefaultWebSecurityManager securityManager() {

        DefaultWebSecurityManager securityManager = new DefaultWebSecurityManager();
        // 配置shiro使用的realm。
        securityManager.setRealm(userRealm());
        // 配置sessionManager
        securityManager.setSessionManager(sessionManager());
        return securityManager;
    }

    /**
     * 配置shiro工厂。
     *
     * @return
     */
    @Bean
    public ShiroFilterFactoryBean shiroFilter() {

        CustomShiroFilterFactoryBean shiroFilterFactory = new CustomShiroFilterFactoryBean();
        shiroFilterFactory.setSecurityManager(securityManager());
        shiroFilterFactory.setLoginUrl("/login");
        // 设置访问过滤条件。
        LinkedHashMap<String, String> map = new LinkedHashMap<String, String>();
        map.put("/static/**", "anon");
        map.put("/login", "anon");
        map.put("/upPass", "anon");
        map.put("/**", "user");
        shiroFilterFactory.setFilterChainDefinitionMap(map);
        return shiroFilterFactory;
    }

    /**
     * 开启shiro注解
     *
     * @return
     */
    @Bean
    public AuthorizationAttributeSourceAdvisor authorizationAttributeSourceAdvisor(
            @Qualifier("securityManager") SecurityManager securityManager) {
        AuthorizationAttributeSourceAdvisor authorizationAttributeSourceAdvisor = new AuthorizationAttributeSourceAdvisor();
        authorizationAttributeSourceAdvisor.setSecurityManager(securityManager);
        return authorizationAttributeSourceAdvisor;
    }

    @Bean
    public DefaultAdvisorAutoProxyCreator defaultAdvisorAutoProxyCreator() {

        DefaultAdvisorAutoProxyCreator advisorProxyCreator = new DefaultAdvisorAutoProxyCreator();
        advisorProxyCreator.setProxyTargetClass(true);
        return advisorProxyCreator;
    }
}
